#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/io.h>
#include <avr/pgmspace.h>

#include "lcd.h"

#define SIZE 3

uint32_t score=0;
/*void drawBoard(int board[SIZE][SIZE]) {
	int x,y;
	char c;

	lcd_goto_xy(1, 25);
	lcd_chr(score+'0');
	
	for (y=0; y<SIZE; y++) {
		for (x=0;x<SIZE;x++) {
		}
	}
}*/
int countEmpty(int board[SIZE][SIZE]) ;
int findTarget(int array[SIZE], int x, int stop) {
	int t;
	// if the position is already on the first, don't evaluate
	if (x == 0) {
		return x;
	}
	for(t = x-1; t >= 0; t--) {
		if (array[t] != 0) {
			if (array[t] != array[x]) {
				// merge is not possible, take next position
				return t+1;
			}
			return t;
		} else {
			// we should not slide further, return this one
			if (t == stop) {
				return t;
			}
		}
	}
	// we did not find a
	return x;
}

int slideArray(int array[SIZE]) {
	int success = 0;
	int x,t,stop=0;

	for (x=0; x<SIZE; x++) {
		if (array[x] != 0) {
			t = findTarget(array, x, stop);
			// if target is not original position, then move or merge
			if (t != x) {
				// if target is zero, this is a move
				if (array[t] == 0) {
					array[t] = array[x];
				} else if (array[t] == array[x]) {
					// merge (increase power of two)
					array[t]+= array[x];
					// increase score
					score += (uint32_t)1<<array[t];
					// set stop to avoid double merge
					stop = t+1;
				}
				array[x]=0;
				success = 1;
			}
		}
	}
	return 1;
}

void rotateBoard(int board[SIZE][SIZE]) {
	int i,j,n=SIZE;
	int tmp;
	for (i=0; i<n/2; i++) {
		for (j=i; j<n-i-1; j++) {
			tmp = board[i][j];
			board[i][j] = board[j][n-i-1];
			board[j][n-i-1] = board[n-i-1][n-j-1];
			board[n-i-1][n-j-1] = board[n-j-1][i];
			board[n-j-1][i] = tmp;
		}
	}
}
int addRandom(int board[SIZE][SIZE]);

int moveUp(int board[SIZE][SIZE]) {
	int success = 0;
	int x;
	for (x=0;x<SIZE;x++) {
		success = slideArray(board[x]);
	}
	
	return 1;
}

int moveLeft(int board[SIZE][SIZE]) {
	int success;
	int x;

	for (x=0;x<SIZE;x++) {
		success = slideArray(board[x]);
	}
	int ret = 0;
	while(ret == 0) {
		addRandom(board);
	}
	return success;
}

int moveDown(int board[SIZE][SIZE]) {
	int success;
	
	rotateBoard(board);
	rotateBoard(board);
	success = moveUp(board);
	rotateBoard(board);
	rotateBoard(board);
	
	return success;
}

int moveRight(int board[SIZE][SIZE]) {
	int success;
	
	rotateBoard(board);
	rotateBoard(board);
	rotateBoard(board);
	success = moveUp(board);
	rotateBoard(board);
	rotateBoard(board);
	
	return success;
}

int findPairDown(int board[SIZE][SIZE]) {
	int success = 0;
	int x,y;
	for (x=0;x<SIZE;x++) {
		for (y=0;y<SIZE-1;y++) {
			if (board[x][y]==board[x][y+1]) return 1;
		}
	}
	return success;
}

int countEmpty(int board[SIZE][SIZE]) {
	int x,y;
	int count=0;
	for (x=0;x<SIZE;x++) {
		for (y=0;y<SIZE;y++) {
			if (board[x][y]==0) {
				count++;
			}
		}
	}
	return count;
}
int checkSuccess(int board[SIZE][SIZE]){
	int x,y;
	
	for (x=0;x<SIZE;x++) {
		for (y=0;y<SIZE;y++) {
			if (board[x][y] == 2048) {
				return 1;
			}
		}
	}
	return 0;
	
}
int gameEnded(int board[SIZE][SIZE]) {
	int ended = 1;
	if (checkSuccess(board) == 1) return 10;
	if (countEmpty(board)>0) return 0;
	if (findPairDown(board)) return 0;
	rotateBoard(board);
	if (findPairDown(board)) ended = 0;
	rotateBoard(board);
	rotateBoard(board);
	rotateBoard(board);
	return ended;
}

int addRandom(int board[SIZE][SIZE]) {
	
	int r = rand()%2;
	if(r == 0)
		for(int i = 0; i < SIZE; i++)
			for(int j = 0; j < SIZE; j++){
				if(board[i][j] == 0) {
					board[i][j] = 2;
					return 1;
				}					
			}
	else if (r == 1)
		for(int i = 0; i < SIZE; i++)
			for(int j = 0; j < SIZE; j++){
				if(board[i][j] == 0) {
					board[i][j] = 4;
					return 1;
				}
			}	
	
	return 0;
}

void initBoard(int board[SIZE][SIZE]) {
	int x,y;
	for (x=0; x<SIZE; x++) {
		for (y=0; y<SIZE; y++) {
			board[x][y]=0;
		}
	}
	board[0][0] = 2;
	board[0][1] = 2;
	board[1][1] = 4;
	board[1][2] = 4;

	score = 0;
}