/*
** lcd.h
**
** LCD 3310 driver
** Target: ATMEGA :: AVR-GCC
*/
#ifndef _NOKIALCD_H_
#define _NOKIALCD_H_

/* Lcd screen size */
#define LCD_X_RES 84
#define LCD_Y_RES 48
#define LCD_CACHE_SIZE ((LCD_X_RES * LCD_Y_RES) / 8)

/* Pinout for LCD */
#define LCD_CLK_PIN 	(1<<PC6)
#define LCD_DATA_PIN 	(1<<PC5)
#define LCD_DC_PIN 		(1<<PC4)
#define LCD_CE_PIN 		(1<<PC3)
#define LCD_RST_PIN 	(1<<PC2)
#define LCD_PORT		PORTC
#define LCD_DDR			DDRC


/* Function for my special characters */
#define	ICON(x)		'z'+1+x

/* Command type sent to the lcd */
typedef enum { LCD_CMD  = 0, LCD_DATA = 1 } LcdCmdData;

void lcd_init(void);
void lcd_contrast(unsigned char contrast);
void lcd_clear(void);
void lcd_clear_area(unsigned char line, unsigned char startX, unsigned char endX);
void lcd_clear_line(unsigned char line);
void lcd_goto_xy(unsigned char x, unsigned char y);
void lcd_goto_xy_exact(unsigned char x, unsigned char y);
void lcd_chr(char chr);
void lcd_str(char* str);


void reset_pixel(unsigned char i,unsigned char j);
void set_pixel(unsigned char i,unsigned char j);

void lcd_base_addr(unsigned int addr);
void lcd_send(unsigned char data, LcdCmdData cd);
#endif